@@ -1,6 +1,6 @@
 /*  Plzip - Parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009-2015 Antonio Diaz Diaz.
+    Copyright (C) 2009-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ void Pretty_print::operator()( const char * const msg ) const
       {
       first_post = false;
       std::fprintf( stderr, "  %s: ", name_.c_str() );
-      for( unsigned i = 0; i < longest_name - name_.size(); ++i )
+      for( unsigned i = name_.size(); i < longest_name; ++i )
         std::fputc( ' ', stderr );
       if( !msg ) std::fflush( stderr );
       }
@@ -213,16 +213,16 @@ extern "C" void * dworker( void * arg )
     // start the workers and wait for them to finish.
 int decompress( int num_workers, const int infd, const int outfd,
                 const Pretty_print & pp, const int debug_level,
-                const bool infd_isreg )
+                const bool ignore_trailing, const bool infd_isreg )
   {
   if( !infd_isreg )
-    return dec_stream( num_workers, infd, outfd, pp, debug_level );
+    return dec_stream( num_workers, infd, outfd, pp, debug_level, ignore_trailing );
 
-  const File_index file_index( infd );
+  const File_index file_index( infd, ignore_trailing );
   if( file_index.retval() == 1 )
     {
     lseek( infd, 0, SEEK_SET );
-    return dec_stream( num_workers, infd, outfd, pp, debug_level );
+    return dec_stream( num_workers, infd, outfd, pp, debug_level, ignore_trailing );
     }
   if( file_index.retval() != 0 )
     { pp( file_index.error().c_str() ); return file_index.retval(); }
@@ -266,15 +266,15 @@ int decompress( int num_workers, const int infd, const int outfd,
   delete[] worker_threads;
   delete[] worker_args;
 
-  const unsigned long long in_size = file_index.file_end();
-  const unsigned long long out_size = file_index.data_end();
+  const unsigned long long in_size = file_index.cdata_size();
+  const unsigned long long out_size = file_index.udata_size();
   if( verbosity >= 2 && out_size > 0 && in_size > 0 )
     std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, %5.2f%% saved.  ",
                   (double)out_size / in_size,
                   ( 8.0 * in_size ) / out_size,
                   100.0 * ( 1.0 - ( (double)in_size / out_size ) ) );
   if( verbosity >= 4 )
-    std::fprintf( stderr, "decompressed size %9llu, size %9llu.  ",
+    std::fprintf( stderr, "decompressed %9llu, compressed %9llu.  ",
                   out_size, in_size );
 
   if( verbosity >= 1 ) std::fputs( (outfd < 0) ? "ok\n" : "done\n", stderr );
