@@ -1,5 +1,5 @@
 /*  Plzip - Parallel compressor compatible with lzip
-    Copyright (C) 2009-2015 Antonio Diaz Diaz.
+    Copyright (C) 2009-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,26 +50,31 @@ class File_index
 
   std::vector< Member > member_vector;
   std::string error_;
+  const long long isize;
   int retval_;
 
   void set_errno_error( const char * const msg );
-  void set_num_error( const char * const msg1, unsigned long long num,
-                      const char * const msg2 = "" );
+  void set_num_error( const char * const msg, unsigned long long num );
+  bool skip_trailing_data( const int fd, long long & pos );
 
 public:
-  explicit File_index( const int infd );
+  File_index( const int infd, const bool ignore_trailing );
 
   long members() const { return member_vector.size(); }
   const std::string & error() const { return error_; }
   int retval() const { return retval_; }
 
-  long long data_end() const
-    { if( member_vector.size() ) return member_vector.back().dblock.end();
-      else return 0; }
+  long long udata_size() const
+    { if( member_vector.empty() ) return 0;
+      return member_vector.back().dblock.end(); }
 
-  long long file_end() const
-    { if( member_vector.size() ) return member_vector.back().mblock.end();
-      else return 0; }
+  long long cdata_size() const
+    { if( member_vector.empty() ) return 0;
+      return member_vector.back().mblock.end(); }
+
+  // total size including trailing data (if any)
+  long long file_size() const
+    { if( isize >= 0 ) return isize; else return 0; }
 
   const Block & dblock( const long i ) const
     { return member_vector[i].dblock; }
